-- 1 up
create sequence "uid_seq" start 100000;

create table "users" (
    "id" integer not null default nextval('uid_seq'),
    "uuid" uuid not null,
    "octid" integer not null,
    "username" text,
    "is_active" bool not null default true,
    "displayname" text,
    "notes" text,
    "properties" text,
    primary key("id"),
    unique("uuid"),
    unique("username")
);

create table "agents" (
    "id" integer not null default nextval('uid_seq'),
    "principal_id" integer not null,
    "agent_id" integer not null,
    "is_active" bool not null default true,
    "function" text,
    primary key("id"),
    foreign key("principal_id") references "users" ("id") on update cascade on delete restrict,
    foreign key("agent_id") references "users" ("id") on update cascade on delete restrict
);

create table "functions" (
    "id" integer not null default nextval('uid_seq'),
    "user_id" integer not null,
    "kind_id" integer not null default 0,
    "is_active" bool not null default true,
    "begin" date,
    "end" date,
    "name" text not null,
    "description" text,
    "url" text,

    "income_reason" text not null,
    "income_amount" numeric not null,
    "income_currency" char(3) not null default 'CZK',
    "income_is_brutto" boolean not null default false,

    "income_autofill" boolean not null default false,
    "income_autoapprove" boolean not null default false,

    "properties" text,
    primary key("id"),
    foreign key("user_id") references "users" ("id") on update cascade on delete restrict
);

create table "grants" (
    "id" integer not null default nextval('uid_seq'),
    "agent_id" integer not null,
    "function_id" integer not null,
    "is_active" bool not null default true,
    "grants" text,
    primary key("id"),
    foreign key("agent_id") references "agents" ("id") on update cascade on delete restrict,
    foreign key("function_id") references "functions" ("id") on update cascade on delete cascade
);

create table "profits" (
    "id" integer not null default nextval('uid_seq'),
    "user_id" integer not null,
    "agent_id" integer,
    "function_id" integer,
    "created" timestamp(0) not null default now(),
    "deleted" timestamp(0),

    "year" integer not null,
    "month" integer not null,

    "reason" text not null,
    "amount" numeric not null,
    "currency" char(3) not null default 'CZK',
    "is_brutto" boolean not null default false,
    "is_approved" boolean not null default false,

    "notes" text,
    primary key("id"),
    foreign key("user_id") references "users" ("id") on update cascade on delete restrict,
    foreign key("agent_id") references "agents" ("id") on update cascade on delete restrict,
    foreign key("function_id") references "functions" ("id") on update cascade on delete set null
);
